/* Estabelece conexo com o banco */

CONNECT 'C:\Livros\Firebird\Projetos\Activity.gdb' 
  USER 'SYSDBA' PASSWORD 'masterkey';

/* Desativa commit automtico */

SET AUTODDL OFF;

/* Define domnios */

CREATE DOMAIN dom_Nome AS CHAR(50);
CREATE DOMAIN dom_Data AS DATE DEFAULT 'NOW';

/* Cria tabelas */

CREATE TABLE Departamento (
  Nu_Departamento INTEGER NOT NULL,
  Sigla CHAR(20) NOT NULL,
  No_Departamento dom_Nome,
  Categoria CHAR(8) DEFAULT 'Local' NOT NULL,
  Telefone CHAR(16),
  Nu_Recurso INTEGER,
  Nu_Depto_Superior INTEGER,
CHECK (Categoria IN ('Central', 'Regional', 'Local')),
CHECK ((Nu_Depto_Superior > 0) OR (Nu_Depto_Superior IS NULL)),
CONSTRAINT UNIQUE_SIGLA 
UNIQUE (Sigla),
CONSTRAINT PK_DEPARTAMENTO 
PRIMARY KEY (Nu_Departamento)
);

CREATE TABLE Cargo (
  Nu_Cargo INTEGER NOT NULL,
  No_Cargo CHAR(30),
  Valor NUMERIC(15,2),
CONSTRAINT PK_CARGO 
PRIMARY KEY (Nu_Cargo)
);

CREATE TABLE Recurso (
  Nu_Recurso INTEGER NOT NULL,
  No_Recurso dom_Nome,
  Situacao CHAR(7) DEFAULT 'Ativo' NOT NULL,
  Nu_Departamento INTEGER,
  Nu_Cargo INTEGER,
CHECK (Situacao in ('Ativo', 'Inativo')),
CONSTRAINT PK_RECURSO 
PRIMARY KEY (Nu_Recurso)
);

CREATE TABLE Projeto (
  Nu_Departamento INTEGER NOT NULL,
  Nu_Projeto INTEGER NOT NULL,
  Sg_Projeto CHAR(10) NOT NULL,
  No_Projeto CHAR(50),
  Dt_Criacao dom_Data,
  Dt_Inicio DATE,
  Dt_Encerramento DATE,
  Situacao CHAR(10) DEFAULT 'Criado' NOT NULL,
  Nu_Recurso INTEGER,
CHECK (Dt_Encerramento IS NULL OR Dt_Encerramento >= Dt_Inicio),
CHECK (Situacao in ('Criado', 'Iniciado', 'Finalizado', 'Cancelado', 'Suspenso')),
CONSTRAINT UNIQUE_SG_PROJETO
UNIQUE (Sg_Projeto),
CONSTRAINT PK_PROJETO 
PRIMARY KEY (Nu_Departamento,Nu_Projeto)
);

CREATE TABLE Atividade (
  Nu_Departamento INTEGER NOT NULL,
  Nu_Projeto INTEGER NOT NULL,
  Nu_Atividade INTEGER NOT NULL,
  No_Atividade dom_Nome,
  Prioridade INTEGER,
  Dt_Inicio DATE,
  Dt_Encerramento DATE,
CHECK (Prioridade between 1 and 5),
CHECK (Dt_Encerramento IS NULL OR Dt_Encerramento >= Dt_Inicio),
CONSTRAINT PK_ATIVIDADE 
PRIMARY KEY (Nu_Departamento,Nu_Projeto,Nu_Atividade)
);

CREATE TABLE Tarefa (
  Nu_Departamento INTEGER NOT NULL,
  Nu_Projeto INTEGER NOT NULL,
  Nu_Atividade INTEGER NOT NULL,
  Nu_Tarefa INTEGER NOT NULL,
  No_Tarefa dom_Nome,
  Dt_Inicio DATE,
  Dt_Encerramento DATE,
CHECK (Dt_Encerramento IS NULL OR Dt_Encerramento >= Dt_Inicio),
CONSTRAINT PK_TAREFA 
PRIMARY KEY (Nu_Departamento,Nu_Projeto,Nu_Atividade,Nu_Tarefa)
);

CREATE TABLE Processo (
  Nu_Departamento INTEGER NOT NULL,
  Nu_Projeto INTEGER NOT NULL,
  Nu_Atividade INTEGER NOT NULL,
  Nu_Tarefa INTEGER NOT NULL,
  Nu_Recurso INTEGER NOT NULL,
  Dt_Inicio dom_Data,
  Dt_Encerramento DATE,
  Detalhe BLOB SUB_TYPE TEXT SEGMENT SIZE 400,
CHECK (Dt_Encerramento IS NULL OR Dt_Encerramento >= Dt_Inicio),
CONSTRAINT PK_PROCESSO
PRIMARY KEY (Nu_Departamento,Nu_Projeto,Nu_Atividade,Nu_Tarefa,Nu_Recurso)
);

CREATE TABLE Dossie (
  Nu_Departamento INTEGER NOT NULL,
  Nu_Projeto INTEGER NOT NULL,
  Nu_Atividade INTEGER NOT NULL,
  Nu_Tarefa INTEGER NOT NULL,
  Nu_Recurso INTEGER NOT NULL,
  Dt_Referencia DATE NOT NULL,
  Horas_Trabalhadas FLOAT,
  Andamento BLOB SUB_TYPE TEXT SEGMENT SIZE 400,
CONSTRAINT PK_DOSSIE
PRIMARY KEY (Nu_Departamento,Nu_Projeto,Nu_Atividade,Nu_Tarefa,Nu_Recurso,Dt_Referencia)
);

/* Cria ndices secundrios */

CREATE INDEX ix_No_Departamento ON Departamento (No_Departamento);
CREATE INDEX ix_No_Cargo        ON Cargo (No_Cargo);
CREATE INDEX ix_No_Recurso      ON Recurso (No_Recurso);
CREATE INDEX ix_No_Projeto      ON Projeto (No_Projeto);
CREATE INDEX ix_No_Tarefa       ON Tarefa (No_Tarefa);

/* Cria vises */

CREATE VIEW VW_Cargo AS
  SELECT Nu_Cargo, No_Cargo
  FROM   Cargo;

/* Cria papis */

CREATE ROLE Grupo1;
CREATE ROLE Grupo2;
CREATE ROLE Grupo3;

/* Define permisses sobre tabelas e vises */

/* Permisses gerais */

GRANT REFERENCES ON Departamento TO PUBLIC;
GRANT REFERENCES ON Cargo        TO PUBLIC;
GRANT REFERENCES ON Recurso      TO PUBLIC;
GRANT REFERENCES ON Projeto      TO PUBLIC;
GRANT REFERENCES ON Atividade    TO PUBLIC;
GRANT REFERENCES ON Tarefa       TO PUBLIC;
GRANT REFERENCES ON Processo     TO PUBLIC;
GRANT REFERENCES ON Dossie       TO PUBLIC;

GRANT SELECT     ON Departamento TO PUBLIC;
GRANT SELECT     ON VW_Cargo     TO PUBLIC;
GRANT SELECT     ON Recurso      TO PUBLIC;
GRANT SELECT     ON Projeto      TO PUBLIC;
GRANT SELECT     ON Atividade    TO PUBLIC;
GRANT SELECT     ON Tarefa       TO PUBLIC;
GRANT SELECT     ON Processo     TO PUBLIC;
GRANT SELECT     ON Dossie       TO PUBLIC;

/* Permisses para o Grupo1 */

GRANT INSERT     ON Departamento TO Grupo1 WITH GRANT OPTION;
GRANT UPDATE     ON Departamento TO Grupo1 WITH GRANT OPTION;
GRANT DELETE     ON Departamento TO Grupo1 WITH GRANT OPTION;

GRANT SELECT     ON Cargo        TO Grupo1 WITH GRANT OPTION;
GRANT INSERT     ON Cargo        TO Grupo1 WITH GRANT OPTION;
GRANT UPDATE     ON Cargo        TO Grupo1 WITH GRANT OPTION;
GRANT DELETE     ON Cargo        TO Grupo1 WITH GRANT OPTION;

GRANT REFERENCES ON VW_Cargo     TO Grupo1 WITH GRANT OPTION;
GRANT INSERT     ON VW_Cargo     TO Grupo1 WITH GRANT OPTION;
GRANT UPDATE     ON VW_Cargo     TO Grupo1 WITH GRANT OPTION;
GRANT DELETE     ON VW_Cargo     TO Grupo1 WITH GRANT OPTION;

GRANT INSERT     ON Recurso      TO Grupo1 WITH GRANT OPTION;
GRANT UPDATE     ON Recurso      TO Grupo1 WITH GRANT OPTION;
GRANT DELETE     ON Recurso      TO Grupo1 WITH GRANT OPTION;

GRANT INSERT     ON Projeto      TO Grupo1 WITH GRANT OPTION;
GRANT UPDATE     ON Projeto      TO Grupo1 WITH GRANT OPTION;
GRANT DELETE     ON Projeto      TO Grupo1 WITH GRANT OPTION;

GRANT INSERT     ON Atividade    TO Grupo1 WITH GRANT OPTION;
GRANT UPDATE     ON Atividade    TO Grupo1 WITH GRANT OPTION;
GRANT DELETE     ON Atividade    TO Grupo1 WITH GRANT OPTION;

GRANT INSERT     ON Tarefa       TO Grupo1 WITH GRANT OPTION;
GRANT UPDATE     ON Tarefa       TO Grupo1 WITH GRANT OPTION;
GRANT DELETE     ON Tarefa       TO Grupo1 WITH GRANT OPTION;

GRANT INSERT     ON Processo     TO Grupo1 WITH GRANT OPTION;
GRANT UPDATE     ON Processo     TO Grupo1 WITH GRANT OPTION;
GRANT DELETE     ON Processo     TO Grupo1 WITH GRANT OPTION;

GRANT INSERT     ON Dossie       TO Grupo1 WITH GRANT OPTION;
GRANT UPDATE     ON Dossie       TO Grupo1 WITH GRANT OPTION;
GRANT DELETE     ON Dossie       TO Grupo1 WITH GRANT OPTION;

/* Permisses para o Grupo2 */

GRANT INSERT     ON Atividade    TO Grupo2;
GRANT UPDATE     ON Atividade    TO Grupo2;
GRANT DELETE     ON Atividade    TO Grupo2;

GRANT INSERT     ON Tarefa       TO Grupo2;
GRANT UPDATE     ON Tarefa       TO Grupo2;
GRANT DELETE     ON Tarefa       TO Grupo2;

GRANT INSERT     ON Processo     TO Grupo2;
GRANT UPDATE     ON Processo     TO Grupo2;
GRANT DELETE     ON Processo     TO Grupo2;

GRANT INSERT     ON Dossie       TO Grupo2;
GRANT UPDATE     ON Dossie       TO Grupo2;
GRANT DELETE     ON Dossie       TO Grupo2;

/* Permisses para o Grupo3 */

GRANT SELECT     ON Projeto      TO Grupo3;
GRANT UPDATE     ON Processo     TO Grupo3;
GRANT INSERT     ON Dossie       TO Grupo3;
GRANT UPDATE     ON Dossie       TO Grupo3;
GRANT DELETE     ON Dossie       TO Grupo3;

/* Salva as alteraes */

COMMIT WORK;

/* Ativa commit automtico */

SET AUTODDL ON;
