/* Estabelece conexo com o banco */

CONNECT 'C:\Livros\Firebird\Projetos\Activity.gdb' 
  USER 'SYSDBA' PASSWORD 'masterkey';

/* Cria excees */

CREATE EXCEPTION departamento_indevido
  'O recurso no pertence ao departamento!';
CREATE EXCEPTION projeto_finalizado
  'Projeto finalizado: os dados no podem ser modificados!';
CREATE EXCEPTION projeto_cancelado
  'Projeto cancelado: os dados no podem ser modificados!';
CREATE EXCEPTION projeto_suspenso
  'Projeto suspenso: os dados no podem ser modificados!';
CREATE EXCEPTION atividade_encerrada
  'Atividade encerrada: os dados no podem ser modificados!';
CREATE EXCEPTION tarefa_encerrada
  'Tarefa encerrada: os dados no podem ser modificados!';
CREATE EXCEPTION processo_encerrado
  'Processo encerrado: os dados no podem ser modificados!';

/* Cria geradores */

CREATE GENERATOR gen_nu_departamento;
CREATE GENERATOR gen_nu_cargo;
CREATE GENERATOR gen_nu_recurso;

/* Desativa commit automtico */

SET AUTODDL OFF;

/* Modifica o terminador de comandos */

SET TERM ^;

/* Define gatilhos (triggers) */

/* Gatilhos na tabela Departamento */

CREATE TRIGGER departamento_get_numero FOR Departamento 
  ACTIVE BEFORE INSERT POSITION 0
AS
  DECLARE VARIABLE nu_departamento INTEGER;
BEGIN
  IF (new.Nu_Departamento IS NOT NULL) THEN
  BEGIN
    SELECT Nu_Departamento
    FROM   Recurso
    WHERE  Nu_Recurso = new.Nu_Recurso
    INTO   :nu_departamento;

    IF (nu_departamento <> new.Nu_Departamento) THEN
      EXCEPTION departamento_indevido;
  END

  IF (new.Nu_Departamento IS NULL) THEN
    new.Nu_Departamento = gen_id( gen_nu_departamento,1 );
END
^

/* Gatilhos na tabela Cargo */

CREATE TRIGGER cargo_get_numero FOR Cargo 
  ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (new.Nu_Cargo IS NULL) THEN
    new.Nu_Cargo = gen_id( gen_nu_cargo,1 );
END 
^   

/* Gatilhos na tabela Recurso */

CREATE TRIGGER recurso_get_numero FOR Recurso 
  ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (new.Nu_Recurso IS NULL) THEN
    new.Nu_Recurso = gen_id( gen_nu_recurso,1 );
END
^

/* Gatilhos na tabela Projeto */

CREATE TRIGGER projeto_get_numero FOR Projeto 
  ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (new.Nu_Projeto IS NULL) THEN
    EXECUTE PROCEDURE projeto_sequencial new.Nu_Departamento
    RETURNING_VALUES  new.Nu_Projeto;
END 
^

CREATE TRIGGER projeto_before_update FOR Projeto
  ACTIVE BEFORE UPDATE POSITION 0
AS
BEGIN
  IF (old.Situacao = 'Finalizado') THEN
    EXCEPTION projeto_finalizado;

  IF (old.Situacao = 'Cancelado') THEN
    EXCEPTION projeto_cancelado;
END
^

/* Gatilhos na tabela Atividade */
  
CREATE TRIGGER atividade_get_numero FOR Atividade 
  ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (new.Nu_Atividade IS NULL) THEN
    EXECUTE PROCEDURE atividade_sequencial new.Nu_Departamento,
                                           new.Nu_Projeto
    RETURNING_VALUES  new.Nu_Atividade;
END 
^   
   
CREATE TRIGGER tarefa_cria FOR Atividade 
  ACTIVE AFTER INSERT POSITION 1
AS
BEGIN
  INSERT INTO Tarefa VALUES (
    new.Nu_Departamento,
    new.Nu_Projeto,
    new.Nu_Atividade,
    null,
    new.No_Atividade,
    null,
    null
  );
END 
^
   
CREATE TRIGGER atividade_before_update FOR Atividade
  ACTIVE BEFORE UPDATE POSITION 0
AS
  DECLARE VARIABLE situacao CHAR(10);
BEGIN
  IF (old.Dt_Encerramento IS NOT NULL) THEN
    EXCEPTION atividade_encerrada;

  situacao = '';

  SELECT Situacao FROM Projeto
  WHERE  Nu_Departamento = new.Nu_Departamento
  AND    Nu_Projeto = new.Nu_Projeto
  INTO   :situacao;

  IF (situacao = 'Finalizado') THEN
    EXCEPTION projeto_finalizado;
  IF (situacao = 'Cancelado') THEN
    EXCEPTION projeto_cancelado;
  IF (situacao = 'Suspenso') THEN
    EXCEPTION projeto_suspenso;
END
^

CREATE TRIGGER atividade_after_update FOR Atividade
  ACTIVE AFTER UPDATE POSITION 1
AS
  DECLARE VARIABLE contador  INTEGER;
  DECLARE VARIABLE nova_data DATE;
BEGIN
  SELECT MIN(Dt_Inicio)
  FROM   Atividade 
  WHERE  Nu_Departamento = new.Nu_Departamento
  AND    Nu_Projeto = new.Nu_Projeto
  INTO   :nova_data;

  UPDATE Projeto
  SET    Dt_Inicio = :nova_data
  WHERE  Nu_Departamento = new.Nu_Departamento
  AND    Nu_Projeto = new.Nu_Projeto;

  contador = 0;

  SELECT COUNT(Nu_Departamento)
  FROM   Atividade
  WHERE  Nu_Departamento = new.Nu_Departamento
  AND    Nu_Projeto = new.Nu_Projeto
  AND    Dt_Encerramento IS NULL
  INTO   :contador;

  IF (contador = 0) THEN
  BEGIN
    SELECT MAX(Dt_Encerramento)
    FROM   Atividade
    WHERE  Nu_Departamento = new.Nu_Departamento
    AND    Nu_Projeto = new.Nu_Projeto
    INTO   :nova_data;

    UPDATE Projeto
    SET    Dt_Encerramento = :nova_data,
           Situacao = 'Finalizado' 
    WHERE  Nu_Departamento = new.Nu_Departamento
    AND    Nu_Projeto = new.Nu_Projeto;
  END
END
^

/* Gatilhos na tabela Tarefa */

CREATE TRIGGER tarefa_get_numero FOR Tarefa
  ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (new.Nu_Tarefa IS NULL) THEN
    EXECUTE PROCEDURE tarefa_sequencial new.Nu_Departamento,
                                        new.Nu_Projeto,
                                        new.Nu_Atividade
    RETURNING_VALUES  new.Nu_Tarefa;
END 
^   

CREATE TRIGGER tarefa_before_update FOR Tarefa
  ACTIVE BEFORE UPDATE POSITION 0
AS
  DECLARE VARIABLE situacao CHAR(10);
BEGIN
  IF (old.Dt_Encerramento IS NOT NULL) THEN
    EXCEPTION tarefa_encerrada;

  situacao = '';

  SELECT Situacao FROM Projeto
  WHERE  Nu_Departamento = new.Nu_Departamento
  AND    Nu_Projeto = new.Nu_Projeto
  INTO   :situacao;

  IF (situacao = 'Finalizado') THEN
    EXCEPTION projeto_finalizado;
  IF (situacao = 'Cancelado') THEN
    EXCEPTION projeto_cancelado;
  IF (situacao = 'Suspenso') THEN
    EXCEPTION projeto_suspenso;
END
^

CREATE TRIGGER tarefa_after_update FOR Tarefa
  ACTIVE AFTER UPDATE POSITION 1
AS
  DECLARE VARIABLE contador  INTEGER;
  DECLARE VARIABLE nova_data DATE;
BEGIN
  nova_data = NULL;

  SELECT MIN(Dt_Inicio)
  FROM   Tarefa
  WHERE  Nu_Departamento = new.Nu_Departamento
  AND    Nu_Projeto = new.Nu_Projeto
  AND    Nu_Atividade = new.Nu_Atividade
  INTO   :nova_data;

  UPDATE Atividade
  SET    Dt_Inicio = :nova_data
  WHERE  Nu_Departamento = new.Nu_Departamento
  AND    Nu_Projeto = new.Nu_Projeto
  AND    Nu_Atividade = new.Nu_Atividade;

  contador = 0;

  SELECT COUNT(Nu_Departamento)
  FROM   Tarefa
  WHERE  Nu_Departamento = new.Nu_Departamento
  AND    Nu_Projeto = new.Nu_Projeto
  AND    Nu_Atividade = new.Nu_Atividade
  AND    Dt_Encerramento IS NULL
  INTO   :contador;

  IF (contador = 0) THEN
  BEGIN
    SELECT MAX(Dt_Encerramento)
    FROM   Tarefa
    WHERE  Nu_Departamento = new.Nu_Departamento
    AND    Nu_Projeto = new.Nu_Projeto
    AND    Nu_Atividade = new.Nu_Atividade
    INTO   :nova_data;

    UPDATE Atividade
    SET    Dt_Encerramento = :nova_data
    WHERE  Nu_Departamento = new.Nu_Departamento
    AND    Nu_Projeto = new.Nu_Projeto
    AND    Nu_Atividade = new.Nu_Atividade;
  END
END
^

/* Gatilhos na tabela Processo */

CREATE TRIGGER processo_before_insert FOR Processo
  ACTIVE BEFORE INSERT POSITION 0
AS
  DECLARE VARIABLE situacao CHAR(10);
BEGIN
  situacao = '';

  SELECT Situacao FROM Projeto
  WHERE  Nu_Departamento = new.Nu_Departamento
  AND    Nu_Projeto = new.Nu_Projeto
  INTO   :situacao;

  IF (situacao = 'Finalizado') THEN
    EXCEPTION projeto_finalizado;
  IF (situacao = 'Cancelado') THEN
    EXCEPTION projeto_cancelado;
  IF (situacao = 'Suspenso') THEN
    EXCEPTION projeto_suspenso;
END
^

CREATE TRIGGER processo_before_update FOR Processo
  ACTIVE BEFORE UPDATE POSITION 0
AS
  DECLARE VARIABLE situacao CHAR(10);
BEGIN
  IF (old.Dt_Encerramento IS NOT NULL) THEN
    EXCEPTION processo_encerrado;

  situacao = '';

  SELECT Situacao FROM Projeto
  WHERE  Nu_Departamento = new.Nu_Departamento
  AND    Nu_Projeto = new.Nu_Projeto
  INTO   :situacao;

  IF (situacao = 'Finalizado') THEN
    EXCEPTION projeto_finalizado;
  IF (situacao = 'Cancelado') THEN
    EXCEPTION projeto_cancelado;
  IF (situacao = 'Suspenso') THEN
    EXCEPTION projeto_suspenso;
END
^

CREATE TRIGGER processo_after_update FOR Processo
  ACTIVE AFTER UPDATE POSITION 1
AS
  DECLARE VARIABLE contador  INTEGER;
  DECLARE VARIABLE nova_data DATE;
BEGIN
  IF (new.Dt_Encerramento IS NOT NULL) THEN
  BEGIN
    INSERT INTO Dossie VALUES (
      new.Nu_Departamento
    , new.Nu_Projeto
    , new.Nu_Atividade
    , new.Nu_Tarefa
    , new.Nu_Recurso
    , new.Dt_Encerramento
    , 0
    , 'Processo concludo...'
    );
  END

  nova_data = NULL;

  SELECT MIN(Dt_Inicio)
  FROM   Processo
  WHERE  Nu_Departamento = new.Nu_Departamento
  AND    Nu_Projeto = new.Nu_Projeto
  AND    Nu_Atividade = new.Nu_Atividade
  AND    Nu_Tarefa = new.Nu_Tarefa
  INTO   :nova_data;

  UPDATE Tarefa
  SET    Dt_Inicio = :nova_data
  WHERE  Nu_Departamento = new.Nu_Departamento
  AND    Nu_Projeto = new.Nu_Projeto
  AND    Nu_Atividade = new.Nu_Atividade
  AND    Nu_Tarefa = new.Nu_Tarefa;

  contador = 0;

  SELECT COUNT(Nu_Departamento)
  FROM   Processo
  WHERE  Nu_Departamento = new.Nu_Departamento
  AND    Nu_Projeto = new.Nu_Projeto
  AND    Nu_Atividade = new.Nu_Atividade
  AND    Nu_Tarefa = new.Nu_Tarefa
  AND    Dt_Encerramento IS NULL
  INTO   :contador;

  IF (contador = 0) THEN
  BEGIN
    SELECT MAX(Dt_Encerramento)
    FROM   Processo
    WHERE  Nu_Departamento = new.Nu_Departamento
    AND    Nu_Projeto = new.Nu_Projeto
    AND    Nu_Atividade = new.Nu_Atividade
    AND    Nu_Tarefa = new.Nu_Tarefa
    INTO   :nova_data;

    UPDATE Tarefa
    SET    Dt_Encerramento = :nova_data
    WHERE  Nu_Departamento = new.Nu_Departamento
    AND    Nu_Projeto = new.Nu_Projeto
    AND    Nu_Atividade = new.Nu_Atividade
    AND    Nu_Tarefa = new.Nu_Tarefa;
  END
END
^

/* Gatilhos na tabela Dossi */

CREATE TRIGGER dossie_before_update FOR Dossie
  ACTIVE BEFORE UPDATE POSITION 0
AS
  DECLARE VARIABLE data_fim DATE;
  DECLARE VARIABLE situacao CHAR(10);
BEGIN
  data_fim = null;

  SELECT Dt_Encerramento
  FROM   PROCESSO
  WHERE  Nu_Departamento = new.Nu_Departamento
  AND    Nu_Projeto = new.Nu_Projeto  
  AND    Nu_Atividade = new.Nu_Atividade  
  AND    Nu_Tarefa = new.Nu_Tarefa  
  AND    Nu_Recurso = new.Nu_Recurso  
  INTO   data_fim;

  IF (data_fim IS NOT NULL) THEN
    EXCEPTION processo_encerrado;

  situacao = '';

  SELECT Situacao FROM Projeto
  WHERE  Nu_Departamento = new.Nu_Departamento
  AND    Nu_Projeto = new.Nu_Projeto
  INTO   :situacao;

  IF (situacao = 'Finalizado') THEN
    EXCEPTION projeto_finalizado;
  IF (situacao = 'Cancelado') THEN
    EXCEPTION projeto_cancelado;
  IF (situacao = 'Suspenso') THEN
    EXCEPTION projeto_suspenso;
END
^

/* Restaura o terminador de comandos */

SET TERM ;^

/* Salva as alteraes */

COMMIT WORK;

/* Ativa commit automtico */

SET AUTODDL ON;
